<?php

namespace Drupal\views_computed_field\Plugin\views\field;

use Drupal\Core\Form\FormStateInterface;
use Drupal\views\Plugin\views\field\FieldPluginBase;
use Drupal\views\Render\ViewsRenderPipelineMarkup;
use Drupal\views\ResultRow;
use Symfony\Component\ExpressionLanguage\ExpressionLanguage;

/**
 * Field handler to compute values based on a formula.
 *
 * @ingroup views_field_handlers
 *
 * @ViewsField("field_computed_field")
 */
class FieldComputedField extends FieldPluginBase {

  /**
   * {@inheritdoc}
   */
  public function query(): void {
    // Intentionally empty to prevent SQL errors.
    // This field is computed in PHP, not in the database.
  }

  /**
   * {@inheritdoc}
   */
  protected function defineOptions() {
    $options = parent::defineOptions();
    $options['formula'] = ['default' => ''];
    $options['hide_empty_fields'] = ['default' => TRUE];
    $options['error_handling'] = ['default' => 'show'];
    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state): void {
    $options = $this->getPreviousFieldLabels();

    $form['formula'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Formula'),
      '#default_value' => $this->options['formula'],
      '#description' => $this->t('Enter a mathematical formula using field machine names. Available fields: @fields. Example: (field_price * field_quantity) - field_discount',
        ['@fields' => implode(', ', array_keys($options))]),
      '#required' => TRUE,
    ];

    $form['hide_empty_fields'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Hide empty fields'),
      '#default_value' => $this->options['hide_empty_fields'],
      '#description' => $this->t('Treat empty field values as zero in calculations'),
    ];

    $form['error_handling'] = [
      '#type' => 'select',
      '#title' => $this->t('Error handling'),
      '#options' => [
        'show' => $this->t('Show error message'),
        'zero' => $this->t('Return zero on error'),
        'hide' => $this->t('Hide completely'),
      ],
      '#default_value' => $this->options['error_handling'],
    ];

    parent::buildOptionsForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function render(ResultRow $values) {
    try {
      return $this->evaluateFormula();
    }
    catch (\Exception $e) {
      return $this->handleError($e->getMessage());
    }
  }

  /**
   * Evaluates the formula using Symfony's ExpressionLanguage.
   */
  protected function evaluateFormula() {
    $formula = trim($this->options['formula']);
    if (empty($formula)) {
      throw new \InvalidArgumentException($this->t('No formula provided'));
    }

    $values = $this->getFieldValues();
    $expression = new ExpressionLanguage();
    $this->addMathFunctions($expression);
    try {
      $result = $expression->evaluate($formula, $values);
      return ViewsRenderPipelineMarkup::create($result);
    }
    catch (\Exception $e) {
      throw new \RuntimeException($this->t('Formula error: @error', ['@error' => $e->getMessage()]));
    }
  }

  /**
   * Retrieves and validates field values.
   */
  protected function getFieldValues() {
    $field_handlers = $this->displayHandler->getHandlers('field');
    $previous_fields = $this->getPreviousFieldLabels();
    $values = [];

    preg_match_all('/\b(?!(?:round|ceil|floor|min|max|avg)\b)([a-zA-Z_][a-zA-Z0-9_]*)\b(?!\s*\()/', $this->options['formula'], $matches);
    $fields_in_formula = array_unique($matches[1]);

    foreach ($fields_in_formula as $field) {
      if (!isset($previous_fields[$field])) {
        throw new \InvalidArgumentException($this->t('Invalid field: @field', ['@field' => $field]));
      }

      $handler = $field_handlers[$field] ?? NULL;
      $raw_value = $handler && isset($handler->last_render) ? $handler->last_render : NULL;

      if ($this->options['hide_empty_fields'] && empty($raw_value)) {
        $values[$field] = 0;
        continue;
      }

      $numeric_value = $this->convertToNumeric($raw_value);
      $values[$field] = is_numeric($numeric_value) ? $numeric_value : 0;
    }

    return $values;
  }

  /**
   * Converts field values to numeric types.
   */
  protected function convertToNumeric($value) {
    if (is_object($value) && method_exists($value, '__toString')) {
      $value = $value->__toString();
    }

    if (is_numeric($value)) {
      return $value + 0;
    }

    $clean_value = preg_replace('/[^0-9.,]/', '', (string) $value);
    return is_numeric($clean_value) ? $clean_value + 0 : 0;
  }

  /**
   * Handles error display based on configuration.
   */
  protected function handleError($message) {
    \Drupal::logger('views_computed_field')->error($message);

    switch ($this->options['error_handling']) {
      case 'show':
        return ViewsRenderPipelineMarkup::create($this->t('Calculation error: @error', ['@error' => $message]));

      case 'zero':
        return ViewsRenderPipelineMarkup::create('0');

      case 'hide':
      default:
        return ViewsRenderPipelineMarkup::create('');
    }
  }

  /**
   * Add math functions.
   */
  protected function addMathFunctions(ExpressionLanguage $expression) {
    $expression->register('round', function() {}, function($args, $num, $precision = 0) {
      return round($num, $precision);
    });

    $expression->register('ceil', function() {}, function($args, $num) {
      return ceil($num);
    });
  }

}
